/* SnmpSettings module */
(function universalModuleDefinition(root, factory){
    if (typeof exports === 'object'){
    // CommonJS
        module.exports = factory(require('jquery'));
    } else if (typeof define === 'function' && define.amd){
    // AMD
        define(['jquery'], factory);
    } else {
    // Browser
        root.SnmpSettings = factory(root.jQuery);
    }
}(this, function ($) {
    'use strict';
    /**
     * @param {array} data Form data.
     * @return {undefined}
     */
    function SnmpSettings(data) {
        this.constructor(data);
    }

    $.extend(SnmpSettings.prototype = {
        constructor: function (data) {
            this._data = data;

            this.mainForm = $('#snmp_settings_form');
            this.mainForm.find('#snmp_settings_save').click($.proxy(this.save, this));

            window.validateNetwork = function() {
                return window.customValidateNetwork('0|false', 'snmp_trusted_addr', WMS._t('rules.enterValidValue'));
            };

            this.mainForm.checkForm({
                rules: [ // rule, element_id, message
                    'required,snmp_community,' + WMS._t('rules.CannotBeEmpty'),
                    'is_alpha,snmp_community,' + WMS._t('rules.enterValidValue'),

                    'required,snmp_port,' + WMS._t('rules.CannotBeEmpty'),
                    'digits_only,snmp_port,' + WMS._t('rules.enterNumber'),
                    'range=1-65535,snmp_port,' + WMS._t('rules.enterValidValue') + ' 1-65535',

                    'function,validateNetwork'
                ]
            });

            $(window).bind('beforeunload', $.proxy(function () {
                if (this.isDirty()) {
                    return WMS._t('prefs.are_you_sure_want_to_exit_without_save');
                }
                return undefined;
            }, this));
        },

        load: function () {
            if (this._data) {
                this.setData(this._data);
            }
            return this;
        },

        validate: function () {
            return this.mainForm.checkForm('validate');
        },

        getData: function () {
            var data = {};
            this.mainForm.find('input,textarea').not('[type=button]').each(function () {
                var name = $(this).attr('id');
                if (!name) {
                    return;
                }
                name = name.split('snmp_')[1];

                var value = $(this).val();
                if (name == 'enable') {
                    value = $(this).is(':checked') ? 'on' : 'off';
                }
                if ($(this).is('textarea')) {
                    value = $(this).val().split('\n');
                }
                data[name] = value;
            });
            return data;
        },

        setData: function (data) {
            $.each(data, $.proxy(function (name, value) {
                if (Object.isArray(value)) {
                    value = value.join('\n');
                }
                if (name == 'enable') {
                    if (this.mainForm.find('#snmp_' + name).prop('checked') != (value === 'on')) {
                        this.mainForm.find('#snmp_' + name).prop('checked', value === 'on');
                    }
                } else {
                    if (this.mainForm.find('#snmp_' + name).val() != value) {
                        this.mainForm.find('#snmp_' + name).val(value);
                    }
                }
            }, this));
        },

        isDirty: function () {
            var result = false;
            $.each(this.getData(), $.proxy(function (name, value) {
                if (Object.isArray(value)) {
                    value = value.join('\n');
                }
                var dataValue = this._data[name];
                if (Object.isArray(dataValue)) {
                    dataValue = dataValue.join('\n');
                }
                if (dataValue !== value) {
                    result = true;
                    return false;
                }
                return undefined;
            }, this));
            return result;
        },

        save: function () {
            var tnets = $.trim($('#snmp_trusted_addr').val()).split('\n');
            var nets = [];
            tnets.each(function (net) {
                if ($.trim(net) !== '') {
                    nets.push(net);
                }
            });
            $('#snmp_trusted_addr').val(nets.join('\n'));

            if (!this.validate()) {
                return;
            }

            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/snmp/',
                type: 'POST',
                dataType: 'json',
                data: {
                    data: this.getData(),
                    _METHOD: 'PUT'
                },
                success: $.proxy(function (response) {
                    ap_showWaitMessage('waitDiv', 1);
                    $.ajax({
                        url: 'api/v1/PBX/settings/snmp/',
                        type: 'GET',
                        cache: false,
                        dataType: 'json',
                        success: $.proxy(function (response2) {
                            if (response2 && response2.result) {
                                if (response2.result != this._data) {
                                    this._data = response2.result;
                                    this.load();
                                } else {
                                    this._data = response2.result;
                                }
                            } else {
                                this._data = this.getData();
                            }
                        }, this),
                        complete: function () {
                            ap_showWaitMessage('waitDiv', 0);
                        }
                    });
                }, this),
                error: $.proxy(function (jqXHR) {
                    setError('error', this.parseResponseError(jqXHR));
                }, this),
                complete: function () {
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },

        parseResponseError: function (jqXHR) {
            var error = '';
            if (jqXHR && jqXHR.responseText && jqXHR.responseText.indexOf('{') === 0) {
                var response = jQuery.parseJSON(jqXHR.responseText);
                if (response && response.reason) {
                    error = String(response.reason);
                }
            }
            return error || 'Unknown error';
        }

    });

    return SnmpSettings;
}));
